<?php

namespace App\Http\Controllers;

use App\Models\Business;
use App\Models\Buyer;
use App\Models\Seller;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;

class DashboardController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index()
    {
        $user = null;

        if (Auth::guard('seller')->check()) {
            $user = Auth::guard('seller')->user();
            $businesses = $user->businesses;
        } elseif (Auth::guard('buyer')->check()) {
            $user = Auth::guard('buyer')->user();
            $businesses = DB::table('buyers')
                ->where('id', $user->id)
                ->get();
            // echo '<pre>';print_r($businesses);exit;
        }

        if (!$user) {
            return redirect()->route('login.index');
        }

        $countryName = DB::table('countries')->where('id', $user->country)->value('name');

        return view('dashboard.dashboard', compact('user', 'businesses', 'countryName'));
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        $user = null;

        if (Auth::guard('seller')->check()) {
            $user = Auth::guard('seller')->user();
        } elseif (Auth::guard('buyer')->check()) {
            $user = Auth::guard('buyer')->user();
        }

        if (!$user) {
            return redirect()->route('dashboard.index')->with('error', 'You do not have permission to edit this business.');
        }

        $businessDetails = DB::table('businesses')
            ->join('sellers', 'businesses.seller_id', '=', 'sellers.id')
            ->select(
                'sellers.name as seller_name',
                'businesses.email as business_email',
                'sellers.phone_number as seller_phone',
                'businesses.name as business_name'
            )
            ->where('sellers.id', $user->id)
            ->first();

        $currentDate = Carbon::now()->format('d-m-Y');

        return view('business.add', compact('businessDetails', 'currentDate'));
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $user = null;

        if (Auth::guard('seller')->check()) {
            $user = Auth::guard('seller')->user();
        } elseif (Auth::guard('buyer')->check()) {
            $user = Auth::guard('buyer')->user();
        }

        $request->validate([
            'email' => 'required|string|email|max:255|unique:businesses',
            'images' => 'nullable|array',
            'images.*' => 'image|mimes:jpeg,png,jpg,gif|max:2048',
            'terms_conditions' => 'accepted',
            'signature' => 'required',
        ]);

        if (!$user) {
            return redirect()->route('dashboard.index')->with('error', 'You do not have permission to update this business.');
        }

        $business = new Business();
        $lastBusiness = Business::latest('id')->first();
        if ($lastBusiness && $lastBusiness->listing_id) {
            $lastIdNumber = (int) substr($lastBusiness->listing_id, 2);
            $nextIdNumber = $lastIdNumber + 1;
        } else {
            $nextIdNumber = 1;
        }

        $business->listing_id = 'SL' . str_pad($nextIdNumber, 6, '0', STR_PAD_LEFT);
        $business->seller_id = $user->id;
        $business->name = $request->input('name');
        $business->email = $request->input('email');
        $business->phone_number = $request->input('phone_number');
        $business->location = $request->input('location');
        $business->industry_type = $request->input('industry_type');
        $business->selling_type = $request->input('selling_type');
        $business->entity_type = $request->input('entity_type');
        $business->started_in = $request->input('started_in');
        $business->asking_price = $request->input('asking_price');
        $business->operational_status = $request->input('operational_status');
        $business->transaction_deal_type = json_encode($request->input('transaction_deal_type', []));
        $business->business_type = $request->input('business_type');
        $business->clientele_type = $request->input('clientele_type');
        $business->interested_in = $request->input('interested_in');
        $business->sales_turnover = $request->input('sales_turnover');
        $business->employees = $request->input('employees');
        $business->ebitda = $request->input('ebitda');
        $business->posted_by = $request->input('posted_by');
        $business->contact_name = $user->name;
        $business->asking_price_includes = $request->input('asking_price_includes');
        $business->expires_on = $request->input('expires_on');
        $business->premises = $request->input('premises');
        $business->owned_size = $request->input('owned_size');
        $business->owned_market_value = $request->input('owned_market_value');
        $business->lease_rent_per_month = $request->input('lease_rent_per_month');
        $business->security_deposit = $request->input('security_deposit');
        $business->carpet_area = $request->input('carpet_area');
        $business->deal_title = $request->input('deal_title');
        $business->other_premises_info = $request->input('other_premises_info');
        $business->reason_for_sell = $request->input('reason_for_sell');
        $business->other_details = $request->input('other_details');
        $business->instagram_url = $request->input('instagram_url');
        $business->facebook_url = $request->input('facebook_url');
        $business->youtube_url = $request->input('youtube_url');
        $business->pinterest_url = $request->input('pinterest_url');
        $business->linkedin_url = $request->input('linkedin_url');
        $business->website_url = $request->input('website_url');
        if ($request->has('signature')) {
            $signatureData = $request->input('signature');
            $signatureData = str_replace('data:image/png;base64,', '', $signatureData);
            $signatureData = str_replace(' ', '+', $signatureData);
            $signatureImage = base64_decode($signatureData);

            $signatureDirPath = public_path('signatures');

            if (!file_exists($signatureDirPath)) {
                mkdir($signatureDirPath, 0755, true);
            }

            $signaturePath = 'signatures/' . $user->id . '.png';
            file_put_contents(public_path($signaturePath), $signatureImage);

            $business->signature = $signaturePath;
        }
        // echo '<pre>';print_r($business);exit;
        $business->save();

        $documentPath = public_path('business_documents/' . $business->id);
        if (!file_exists($documentPath)) {
            mkdir($documentPath, 0755, true);
        }

        $documentFields = ['proof_of_company', 'proof_of_financials', 'proof_of_agreement'];
        foreach ($documentFields as $document) {
            if ($request->hasFile($document)) {
                if ($business->{$document}) {
                    $oldFilePath = public_path($business->{$document});
                    if (file_exists($oldFilePath)) {
                        unlink($oldFilePath);
                    }
                }

                $file = $request->file($document);
                $fileName = time() . '_' . $file->getClientOriginalName();
                $file->move($documentPath, $fileName);
                $business->{$document} = 'business_documents/' . $business->id . '/' . $fileName;
            }
        }

        // echo '<pre>';print_r($business);exit;
        $business->save();

        if ($request->hasFile('images')) {
            $businessImagePath = public_path('business_images/' . $business->id);

            if (!file_exists($businessImagePath)) {
                mkdir($businessImagePath, 0755, true);
            }

            $uploadedImages = [];
            foreach ($request->file('images') as $file) {
                $fileName = time() . '_' . $file->getClientOriginalName();
                $file->move($businessImagePath, $fileName);
                $uploadedImages[] = $business->id . '/' . $fileName;
            }

            $business->images = json_encode($uploadedImages);
            // echo '<pre>';print_r($business);exit;
            $business->save();
        }

        return redirect()->route('dashboard.index')->with('success', 'Business added successfully!');
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit(string $id)
    {
        $user = null;

        if (Auth::guard('seller')->check()) {
            $user = Auth::guard('seller')->user();
        } elseif (Auth::guard('buyer')->check()) {
            $user = Auth::guard('buyer')->user();
        }

        if (!$user) {
            return redirect()->route('login.index');
        }

        $countries = DB::table('countries')->get();
        $states = DB::table('states')->get();

        return view('dashboard.edit', compact('user', 'countries', 'states'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request)
    {
        $user = null;
        $table = null;

        if (Auth::guard('seller')->check()) {
            $user = Auth::guard('seller')->user();
            $table = Seller::find($user->id);
        } elseif (Auth::guard('buyer')->check()) {
            $user = Auth::guard('buyer')->user();
            $table = Buyer::find($user->id);
        }

        if (!$user || !$table) {
            return redirect()->route('login.index')->with('error', 'User not found or not logged in.');
        }

        $request->validate([
            'name' => 'required|string|max:255',
            'email' => 'required|email|max:255|unique:' . ($table instanceof Seller ? 'sellers' : 'buyers') . ',email,' . $user->id,
            'phone_number' => 'required|string|max:15',
            'country' => 'required|integer',
            'state' => 'required',
            'address' => 'required|string|max:255',
            'city' => 'required|string|max:100',
            'postalcode' => 'required|string|max:10',
            'password' => 'nullable|min:8',
        ]);

        $table->name = $request->input('name');
        $table->email = $request->input('email');
        $table->phone_number = $request->input('phone_number');
        $table->country = $request->input('country');
        $table->state = $request->input('state');
        $table->address = $request->input('address');
        $table->city = $request->input('city');
        $table->postalcode = $request->input('postalcode');
        $table->password = Hash::make($request->input('password'));
        if ($request->filled('password')) {
            $table->password = Hash::make($request->input('password'));
            $table->mypass = base64_encode($request->input('password'));
        }

        // echo '<pre>';print_r($table);exit;

        $table->save();

        return redirect()->route('dashboard.index')->with('success', 'Details updated successfully!');
    }


    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        //
    }

    public function showBusiness(string $id)
    {
        $business = Business::findOrFail($id);
        $user = null;

        if (Auth::guard('seller')->check()) {
            $user = Auth::guard('seller')->user();
            $images = $business->images ? json_decode($business->images, true) : [];
        } elseif (Auth::guard('buyer')->check()) {
            $user = Auth::guard('buyer')->user();
        }

        if (!$user) {
            return redirect()->route('login.index')->with('error', 'Please log in to view business details.');
        }

        // Optional: Restrict viewing based on ownership or roles if needed.
        if ($user->id !== $business->seller_id && !$user->is_admin) {
            return redirect()->route('dashboard.index')->with('error', 'You do not have permission to view this business.');
        }

        return view('business.show', compact('business', 'images'));
    }

    public function editBusiness(string $id)
    {
        $business = Business::findOrFail($id);
        $user = null;

        if (Auth::guard('seller')->check()) {
            $user = Auth::guard('seller')->user();
        } elseif (Auth::guard('buyer')->check()) {
            $user = Auth::guard('buyer')->user();
        }

        if (!$user || $business->seller_id !== $user->id) {
            return redirect()->route('dashboard.index')->with('error', 'You do not have permission to edit this business.');
        }

        $images = $business->images ? json_decode($business->images, true) : [];
        $business->transaction_deal_type = json_decode($business->transaction_deal_type, true);
        return view('business.edit', compact('business', 'images'));
    }

    public function updateBusiness(Request $request, string $id)
    {
        $user = null;

        if (Auth::guard('seller')->check()) {
            $user = Auth::guard('seller')->user();
        } elseif (Auth::guard('buyer')->check()) {
            $user = Auth::guard('buyer')->user();
        }

        $request->validate([
            'images' => 'nullable|array',
            'images.*' => 'image|mimes:jpeg,png,jpg,gif|max:2048',
        ]);

        $business = Business::findOrFail($id);

        if (!$user || $business->seller_id !== $user->id) {
            return redirect()->route('dashboard.index')->with('error', 'You do not have permission to update this business.');
        }

        $lastBusiness = Business::whereNotNull('listing_id')->latest('id')->first();
        if ($lastBusiness && $lastBusiness->listing_id) {
            $lastIdNumber = (int) substr($lastBusiness->listing_id, 2);
            $nextIdNumber = $lastIdNumber + 1;
        } else {
            $nextIdNumber = 1;
        }

        if (!$business->listing_id) {
            $business->listing_id = 'SL' . str_pad($nextIdNumber, 6, '0', STR_PAD_LEFT);
        }
        $business->seller_id = $user->id;
        $business->name = $request->input('name');
        $business->email = $request->input('email');
        $business->phone_number = $request->input('phone_number');
        $business->location = $request->input('location');
        $business->industry_type = $request->input('industry_type');
        $business->selling_type = $request->input('selling_type');
        $business->entity_type = $request->input('entity_type');
        $business->started_in = $request->input('started_in');
        $business->asking_price = $request->input('asking_price');
        $business->operational_status = $request->input('operational_status');
        $business->transaction_deal_type = json_encode($request->input('transaction_deal_type', []));
        $business->business_type = $request->input('business_type');
        $business->clientele_type = $request->input('clientele_type');
        $business->interested_in = $request->input('interested_in');
        $business->sales_turnover = $request->input('sales_turnover');
        $business->employees = $request->input('employees');
        $business->ebitda = $request->input('ebitda');
        $business->posted_by = $request->input('posted_by');
        $business->contact_name = $user->name;
        $business->asking_price_includes = $request->input('asking_price_includes');
        $business->expires_on = $request->input('expires_on');
        $business->premises = $request->input('premises');
        $business->owned_size = $request->input('owned_size');
        $business->owned_market_value = $request->input('owned_market_value');
        $business->lease_rent_per_month = $request->input('lease_rent_per_month');
        $business->security_deposit = $request->input('security_deposit');
        $business->carpet_area = $request->input('carpet_area');
        $business->deal_title = $request->input('deal_title');
        $business->other_premises_info = $request->input('other_premises_info');
        $business->reason_for_sell = $request->input('reason_for_sell');
        $business->other_details = $request->input('other_details');
        $business->instagram_url = $request->input('instagram_url');
        $business->facebook_url = $request->input('facebook_url');
        $business->youtube_url = $request->input('youtube_url');
        $business->pinterest_url = $request->input('pinterest_url');
        $business->linkedin_url = $request->input('linkedin_url');
        $business->website_url = $request->input('website_url');
        if ($request->hasFile('images')) {
            $businessImagePath = public_path('business_images/' . $business->id);

            if (file_exists($businessImagePath) && is_dir($businessImagePath)) {
                $oldImages = json_decode($business->images, true);
                if ($oldImages) {
                    foreach ($oldImages as $oldImage) {
                        $oldImagePath = public_path('business_images/' . $oldImage);
                        if (file_exists($oldImagePath)) {
                            unlink($oldImagePath);
                        }
                    }
                }
                rmdir($businessImagePath);
            }

            if (!file_exists($businessImagePath)) {
                mkdir($businessImagePath, 0755, true);
            }

            $uploadedImages = [];
            foreach ($request->file('images') as $file) {
                $fileName = time() . '_' . $file->getClientOriginalName();
                $file->move($businessImagePath, $fileName);
                $uploadedImages[] = $business->id . '/' . $fileName;
            }

            $business->images = json_encode($uploadedImages);
        }

        if ($request->hasFile('proof_of_company') || $request->hasFile('proof_of_financials') || $request->hasFile('proof_of_agreement')) {
            $documentPath = public_path('business_documents/' . $business->id);

            if (file_exists($documentPath) && is_dir($documentPath)) {
                $oldDocuments = ['proof_of_company', 'proof_of_financials', 'proof_of_agreement'];
                foreach ($oldDocuments as $docField) {
                    if ($business->{$docField}) {
                        $oldDocPath = public_path($business->{$docField});
                        if (file_exists($oldDocPath)) {
                            unlink($oldDocPath);
                        }
                    }
                }
                rmdir($documentPath);
            }

            if (!file_exists($documentPath)) {
                mkdir($documentPath, 0755, true);
            }

            $documentFields = ['proof_of_company', 'proof_of_financials', 'proof_of_agreement'];
            foreach ($documentFields as $field) {
                if ($request->hasFile($field)) {
                    $file = $request->file($field);
                    $fileName = time() . '_' . $file->getClientOriginalName();
                    $file->move($documentPath, $fileName);
                    $business->{$field} = 'business_documents/' . $business->id . '/' . $fileName;
                }
            }
        }

        // echo '<pre>';print_r($business);exit;
        $business->save();

        return redirect()->route('dashboard.index')->with('success', 'Business updated successfully!');
    }

    public function deleteBusiness(string $id)
    {
        $user = null;

        if (Auth::guard('seller')->check()) {
            $user = Auth::guard('seller')->user();
        } elseif (Auth::guard('buyer')->check()) {
            $user = Auth::guard('buyer')->user();
        }

        $business = Business::findOrFail($id);

        if (!$user || $business->seller_id !== $user->id) {
            return redirect()->route('dashboard.index')->with('error', 'You do not have permission to delete this business.');
        }

        $business->delete();

        return redirect()->route('dashboard.index')->with('success', 'Business deleted successfully!');
    }

    public function showBuyer(string $id)
    {
        $buyer = Buyer::findOrFail($id);
        $user = null;

        if (Auth::guard('seller')->check()) {
            $user = Auth::guard('seller')->user();
        } elseif (Auth::guard('buyer')->check()) {
            $user = Auth::guard('buyer')->user();
        }

        if (!$user) {
            return redirect()->route('login.index')->with('error', 'Please log in to view buyer details.');
        }

        if ($user->id !== $buyer->id && !$user->is_admin) {
            return redirect()->route('dashboard.index')->with('error', 'You do not have permission to view this buyer.');
        }

        return view('buyer.show', compact('buyer'));
    }

    public function editBuyer(string $id)
    {
        $buyer = Buyer::findOrFail($id);
        $user = null;

        if (Auth::guard('seller')->check()) {
            $user = Auth::guard('seller')->user();
        } elseif (Auth::guard('buyer')->check()) {
            $user = Auth::guard('buyer')->user();
        }

        if (!$user) {
            return redirect()->route('dashboard.index')->with('error', 'You do not have permission to edit this buyer.');
        }

        $countries = DB::table('countries')->get();
        $states = DB::table('states')->get();

        $images = $buyer->images ? json_decode($buyer->images, true) : [];
        $buyer->transaction_deal_type = json_decode($buyer->transaction_deal_type, true);
        return view('buyer.edit', compact('buyer', 'countries', 'states'));
    }

    public function updateBuyer(Request $request, string $id)
    {
        $user = null;

        if (Auth::guard('seller')->check()) {
            $user = Auth::guard('seller')->user();
        } elseif (Auth::guard('buyer')->check()) {
            $user = Auth::guard('buyer')->user();
        }

        $request->validate([
            'profile_image' => 'nullable|image|mimes:jpeg,png,jpg,gif|max:2048',
        ]);

        $buyer = Buyer::findOrFail($id);

        if (!$user) {
            return redirect()->route('dashboard.index')->with('error', 'You do not have permission to update this business.');
        }

        $lastBuyer = Buyer::whereNotNull('buying_id')->latest('id')->first();
        if ($lastBuyer && $lastBuyer->buying_id) {
            $lastIdNumber = (int) substr($lastBuyer->buying_id, 2);
            $nextIdNumber = $lastIdNumber + 1;
        } else {
            $nextIdNumber = 1;
        }
        if (!$buyer->buying_id) {
            $buyer->buying_id = 'BB' . str_pad($nextIdNumber, 6, '0', STR_PAD_LEFT);
        }
        $buyer->buyer_investor = $request->input('buyer_investor');
        $buyer->name = $request->input('name');
        $buyer->email = $request->input('email');
        $buyer->password = Hash::make($request->input('password'));
        if ($request->filled('password')) {
            $buyer->password = Hash::make($request->input('password'));
            $buyer->mypass = base64_encode($request->input('password'));
        }
        $buyer->phone_number = $request->input('phone_number');
        $buyer->designation = $request->input('designation');
        $buyer->preferred_industry = json_encode($request->input('preferred_industry', []));
        $buyer->preferred_investment_size = $request->input('preferred_investment_size');
        $buyer->country = $request->input('country');
        $buyer->state = $request->input('state');
        $buyer->address = $request->input('address');
        $buyer->street = $request->input('street');
        $buyer->city = $request->input('city');
        $buyer->postalcode = $request->input('postcode');
        $buyer->locations = json_encode($request->input('locations', []));
        $buyer->experience = $request->input('experience');
        $buyer->investment_rationale = $request->input('investment_rationale');
        $buyer->acquire_invest_business = $request->input('acquire_invest_business');
        $buyer->source_of_financing = $request->input('source_of_financing');
        $buyer->posted_by = $request->input('posted_by');
        $buyer->transaction_deal_type = json_encode($request->input('transaction_deal_type', []));
        $buyer->name_of_business = $request->input('name_of_business');
        $buyer->started_in = $request->input('started_in');
        $buyer->employees = $request->input('employees');
        $buyer->website_url = $request->input('website_url');
        $buyer->linkedin_url = $request->input('linkedin_url');
        $buyer->investment_budget = $request->input('investment_budget');
        $buyer->industry_domain_experience = $request->input('industry_domain_experience');
        $buyer->areas_of_expertise = $request->input('areas_of_expertise');
        $buyer->other_info = $request->input('other_info');
        if ($request->hasFile('profile_image')) {
            $profileImagePath = public_path('profile_images/' . $buyer->name);

            if (!file_exists($profileImagePath)) {
                mkdir($profileImagePath, 0755, true);
            }

            if (!empty($buyer->profile_image) && file_exists(public_path($buyer->profile_image))) {
                unlink(public_path($buyer->profile_image));
            }

            $file = $request->file('profile_image');
            $fileName = time() . '_' . $file->getClientOriginalName();
            $file->move($profileImagePath, $fileName);

            $buyer->profile_image = $buyer->name . '/' . $fileName;
        }

        // echo '<pre>';print_r($buyer);exit;
        $buyer->save();

        return redirect()->route('dashboard.index')->with('success', 'Buyer updated successfully!');
    }
}
